#!/usr/bin/env node
const fs = require('fs')
const path = require('path')
const parser = require('@babel/parser')
const traverse = require('@babel/traverse').default
const { transformFromAst } = require('@babel/core')
const options = require('./webapck.config')
const Parser = {
  getAst: path => {
    const content = fs.readFileSync(path, 'utf-8')
    console.log(content)
    return parser.parse(content, {
      sourceType: 'module'
    })
  },
  getDependencies: (ast, fileName) => {
    const dependecies = {}

    traverse(ast, {
      ImportDeclaration ({node}) {
        const dirname = path.dirname(fileName)

        const filepath = './' + path.join(dirname, node.source.value)

        dependecies[node.source.value] = filepath
      }
    })

    return dependecies
  },
  getCode: (ast) => {
    const { code } = transformFromAst(ast, null, {
      presets: ['@babel/preset-env']
    })
    return code
  }
}

class Compiler {
  constructor (options) {
    const { entry, output } = options

    this.entry = entry

    this.output = output

    this.modules = []
  }
  run () {
    const info = this.build(this.entry)
    // console.log(info)
    this.modules.push(info)
    this.modules.forEach(({dependencies}) => {
      if (dependencies) {
        for (const dependency in dependencies) {
          console.log(dependencies[dependency])
          this.modules.push(this.build(dependencies[dependency]))
        }
      }
    })
    const dependencyGraph = this.modules.reduce(
      (graph, item) => ({
        ...graph,
        [item.filename] : {
          dependencies: item.dependencies,
          code: item.code
        }
      }),
      {}
    )
    // console.log(dependencyGraph);
    
    this.generate(dependencyGraph)

  }
  build (filename) {
    const { getAst, getDependencies, getCode } = Parser
    const ast = getAst(filename)
    console.log(ast)
    const dependencies = getDependencies(ast, filename)
    const code = getCode(ast)
    return {
      filename,
      dependencies,
      code
    }
  }
  generate (code) {
    const filePath = path.join(this.output.path, this.output.filename)

    const bundle = `
    (function(graph){
      function require(module){
        function localRequire(relativePath){
          return require(graph[module].dependencies[relativePath])
        }
        var exports = {};
        (function(require,exports,code){
          eval(code)
        })(localRequire,exports,graph[module].code);
        return exports;
      }
      require('${this.entry}')
    })(${JSON.stringify(code)})`

    fs.writeFileSync(filePath, bundle, 'utf-8')
  }
}
new Compiler(options).run()